--[[*

Generates report on database contents.

@author foosion
@requires foo_lua 0.2
@contextcommand Scripting/Generate HTML
]]

require "lhtml.lua"

prefs = {}
local defaults = {}

-- get a unique value
IGNORE = {}

defaults.choose_template_file = 'yes'
defaults.template_file = nil
defaults.choose_output_file = 'yes'
defaults.output_file = nil

local function set_defaults()
	for k, v in pairs(defaults) do
		if not prefs[k] then
			prefs[k] = v
		end
	end
end

local function loadprefs()
	dofile(_SCRIPTDIR..'albumlist-prefs.lua')
end

-- read in preferences
local res, err = pcall(loadprefs)
if not res then
	fb2k.console.error('error while reading preferences:')
	fb2k.console.error(err)
	fb2k.console.info('using default preferences')
end

set_defaults()

-----------------------------
-- builtin template
-----------------------------

local builtin_template = [[Tracks by codec [count]
<<for _, codec in ipairs(codecs_by_count) do>>
<(codec.name)>: <(codec.count)> (<(codec.percent_count)>%)
<<end>>
total: <(total_count)> (100.0%)

Tracks by codec [duration]
<<for _, codec in ipairs(codecs_by_duration) do>>
<(codec.name)>: <(format_duration(codec.duration))> (<(codec.percent_duration)>%)
<<end>>
total: <(format_duration(total_duration))> (100.0%)
]]

-----------------------------
-- prepare data
-----------------------------

tracks = arg[1]

-----------------------------
-- generate output
-----------------------------

local template
if prefs.choose_template_file == 'builtin' then
	fb2k.console.info('using builtin template')
	template = builtin_template
	if not template then
		fb2k.console.error('no builtin template available')
	end
else
	local template_file = prefs.template_file
	if prefs.choose_template_file == 'yes' or not prefs.template_file then
		template_file = dialog.choose_file('LHTML Files|*.lhtml|All Files|*.*', --ext_mask
			1, -- def_ext_mask
			'lhtml', -- def_ext
			'Load template file', -- title
			template_file, -- path
			false -- save
			)
	end
	if template_file then
		fb2k.console.info('opening template file for reading:')
		fb2k.console.info('location: "'..template_file..'"')
		local s = file.open(template_file, stream.MODE_READ)
		if s then
			local len = s:get_length()
			template = s:read(len)
			s:release()
		else
			fb2k.console.error('could not open file')
		end
	end
end
if template then
	fb2k.console.info('compiling template')
	local f, err = lhtml.compile(template)
	
	if f then
		fb2k.console.info('creating document from template')
		text = f()
	else
		error(err)
	end
	
	local output_file = prefs.output_file
	if prefs.choose_output_file == 'yes' or not prefs.output_file then
		output_file = dialog.choose_file('HTML Files|*.html|Text Files|*.txt|All Files|*.*', --ext_mask
			1, -- def_ext_mask
			'html', -- def_ext
			'Save file', -- title
			output_file, -- path
			true -- save
			)
	end
	if output_file then
		fb2k.console.info('opening document file for writing:')
		fb2k.console.info('location: "'..output_file..'"')
		local s = file.open(output_file, stream.MODE_WRITE_NEW)
		if s then
			s:write(text)
			s:release()
		else
			fb2k.console.error('could not open file')
		end
	end
end
