--[[
common.lua

provides common functionality for test scripts,
uses base, table, and fb2k.console libraries
]]

function table.quote(x)
    local sep = ""
    local s = "{"
    for i,v in ipairs(x) do
        s = s .. sep .. quote(v)
        sep = ", "
    end
    if sep ~= "" then
        sep = "; "
    end
    for k,v in pairs(x) do
        if type(k) ~= "number" then
            s = s .. sep .. "[" .. quote(k) .. "] = " .. quote(v)
        end
        sep = ", "
    end
    s = s .. "}"
    return s
end

function string.quote(x)
    return string.format("%q", x)
end

function quote(x)
    local tp = type(x)
    local f = _G[tp] and _G[tp].quote
    if f then
        return f(x)
    else
        return tostring(x)
    end
end

function table.equiv(x, y)
	if type(y) ~= "table" then return false end
	for k, v in x do
		if not equiv(v, y[k]) then return false end
	end
	return true
end

function equiv(x, y)
    local tp = type(x)
    local f = _G[tp] and _G[tp].equiv
    if f then
        return f(x, y)
    else
		tp = type(y)
		f = _G[tp] and _G[tp].equiv
		if f then
			return f(y, x)
		else
			return x == y
		end
    end
end

-------------------------------------------------
-- additions to dialog library
-------------------------------------------------

if dialog then
	local button_to_string = {
		[dialog.IDABORT]	= "abort",
		[dialog.IDCANCEL]	= "cancel",
		[dialog.IDIGNORE]	= "ignore",
		[dialog.IDNO]		= "no",
		[dialog.IDOK]		= "ok",
		[dialog.IDRETRY]	= "retry",
		[dialog.IDYES]		= "yes",
	}
	
	function dialog.yes_or_no(msg, title)
		local D = dialog
		local button = D.messagebox(msg, title, D.MB_YESNO, D.MB_ICONQUESTION)
		return button_to_string[button]
	end
end
