--[[*
Test script for fb2k.playlist.apply_order().

@author foosion
@requires foo_lua 0.2
@contextcommand Scripting/Test/fb2k.playlist.apply_order
]]

assert(fb2k and fb2k.playlist or loadmodule("fb2k.playlist"))

local pl = fb2k.playlist

function assert_apply_order_help(order)
	local before = pl.get_data()
	if table.getn(order) ~= table.getn(before) then return false end
	local expect = {}
	for from, to in ipairs(order) do
		--print(from.." -> "..to)
		expect[from] = before[to]
	end
	pl.apply_order(order)
	local after = pl.get_data()
	if table.getn(order) ~= table.getn(after) then return false end
	for i = 1, table.getn(expect) do
		--print(i, expect[i], after[i])
		if tostring(expect[i]) ~= tostring(after[i]) then return false end
	end
	return true
end

function assert_apply_order(order)
	if assert_apply_order_help(order) then
		print("passed")
	else
		print("failed")
	end
end

local count = pl.get_count()

do
	local order = {n=count}
	for i = 1, count do
		order[i] = i
	end
	assert_apply_order(order)
end

local function f(count, part)
	local order = {n=count}
	if part > count / 2 then part = math.floor(count / 2) end
	for i = 1, part do
		order[i] = count - part + i
		order[count - part + i] = i
	end
	for i = part + 1, count - part do
		order[i] = i
	end
	return order
end

do
	assert_apply_order(f(count, 1))
end

do
	assert_apply_order(f(count, 2))
end

local function g(count, a, b)
	local order = {n=count}
	if a > count then a = count end
	if a + b > count then b = count - a end
	for i = 1, a do
		order[i] = b + i
	end
	for i = 1, b do
		order[a + i] = i
	end
	for i = a + b + 1, count do
		order[i] = i
	end
	return order
end

do
	assert_apply_order(g(count, 3, 5))
end
