-- encoding: UTF-8

function print(...)
	fb2k.console.info(table.concat(arg))
end

local p, s, res

local passed_tests = {}
local failed_tests = {}

function pass_test(name)
	print("passed")
	table.insert(passed_tests,name)
	return true
end

function fail_test(name)
	print("failed")
	table.insert(failed_tests,name)
	return false
end

-- test UTF-8 string
function pattern_test(p, desc, expect)
	assert(expect, "invalid test: expected values missing")
	local name = "pattern test: "..desc
	print(name)
	local s, e = utf8.find(s, p)
	if not s then
		return fail_test(name)
	else
		return pass_test(name)
	end
end

function pattern_test_set(set, ...)
	pattern_test("["..set.."]", "character set '["..set.."]'", unpack(arg))
end

function pattern_test_balanced(c_open, c_close, ...)
	assert(utf8.len(c_open) == 1 and utf8.len(c_open) == 1, "invalid test: delimiters are not single characters")
	pattern_test("%b"..c_open..c_close, "balanced delimiters '"..c_open.."' and '"..c_close.."'", unpack(arg))
end

-- character classes
s = "a	g1.ß_Ä +æ"
fb2k.console.popup()
pattern_test("%a", "letters", "a", "g", "ß", "Ä", "æ")
pattern_test("%d", "digits", "1")
pattern_test("%l", "lowercase letters", "a", "g", "ß", "æ")
pattern_test("%p", "punctuation characters", ".", "_", "+")
pattern_test("%s", "space characters", "	", " ")
pattern_test("%u", "upperase letters", "Ä")
pattern_test("%w", "alphanumeric characters", "a", "g", "1", "ß", "Ä", "æ")
pattern_test("%x", "hexadecimal digits", "a", "1")
pattern_test("%.", "special character '.'", ".")
pattern_test("%Ä", "escaped character 'Ä'", "Ä")

pattern_test_set("a", "a")
pattern_test_set("ag", "a", "g")
pattern_test_set("%a", "a", "g", "ß", "Ä", "æ")
pattern_test_set("Äa-f", "a", "Ä")
pattern_test_set("a-f%Ä", "a", "Ä")
pattern_test_set("^Äa-f", "	", "g", "1", ".", "ß", "_", " ", "+", "æ")

s = "1(2(3ÜÜÜ4)5)6"
pattern_test_balanced("(","Ü", "(2(3ÜÜ")
pattern_test_balanced("Ü",")", "ÜÜ4)5)")

s = "ab\0cd"
pattern_test("%z", "null character", "\0")

print("passed: "..table.getn(passed_tests))
print("failed: "..table.getn(failed_tests))
