--[[*
Preferences for dbstats.lua.

@file dbstats-prefs.lua
]]

-- whether messagebox with stats should be displayed
-- allowed values: 'yes', 'no', nil (for default)
-- default: 'no'
prefs.show_dialog = 'no'

-- whether HTML with stats should be generated
-- allowed values: 'yes', 'no', 'ask', nil (for default)
-- default: 'ask'
prefs.generate_report = 'yes'

-- whether the user should be prompted for a template file
-- if set to 'no', it will always use the default path (see below).
-- if set to 'builtin', the builtin template will be used without further interrogation.
-- allowed values: 'yes', 'no', 'builtin', nil (for default)
-- default: 'yes'
prefs.choose_template_file = 'yes'

-- default path to template
-- allowed values: string with path, nil
-- default: nil
prefs.template_file = _SCRIPTDIR..'Example_1.lhtml'

-- whether the script should ask were to store the generated HTML
-- if set to 'no', it will always use the default path (see below).
-- allowed values: 'yes', 'no', nil (for default)
-- default: 'yes'
prefs.choose_output_file = 'yes'

-- default path to store the generated HTML
-- allowed values: string with path, nil
-- default: nil
prefs.output_file = _SCRIPTDIR..'dbstats.html'

-- If the codec field is missing, the script will try to deduce the codec
-- from the protocol or, if that fails, the file extension.
-- For full details of how the codec is determined, see get_codec().

-- Set up default codec names.
local MP3 = 'MP3'
local APE = "Monkey's Audio"
local MPC = 'Musepack'
local OGG = 'Vorbis'
local FLAC = 'FLAC'
local SHN = 'Shorten'

-- map protocol names to codec names
prefs.protocol_map = {
	-- general format is [key] = val
	-- id = val is short for ['id'] = val, where id is an identifier
	--tone = 'Tone', -- show tone:// URLs as Tone codec
	tone = IGNORE, -- ignore tone:// URLs
	silence = IGNORE,
}

-- map file extensions to codec names
-- IGNORE works here, too.
prefs.extension_map = {
	-- the following two lines map the 'mpc' and 'mp+' extensions to the codec 'Musepack'
	mpc = MPC, -- short for ['mpc'] = MPC
	['mp+'] = MPC, -- we have to use the explicit form since mp+ is no Lua identifier
	mp3 = MP3,
	ogg = OGG,
	ape = APE,
	mac = APE,
	flac = FLAC,
	shn = SHN,
	it = 'Module',
}
