--[[
lhtml.lua
Use with foobar2000+foo_lua or plain Lua 5.0

Testbed for LHTML.
]]

local text = [[<html>
 <head>
  <title>Title</title>
 </head>
 <body>
  <table>
<<for i = 1, total do>>
   <tr>
    <td><(i)></td>
   </tr>
<<end>>
  </table>
 </body>
</html>
]]


-----------------------------
-- buffer class
-----------------------------

buffer = {}

function buffer.create()
	local t = {}
	t.data = {}
	function t:output(...)
		for _,v in ipairs(arg) do
			table.insert(self.data, v)
		end
	end
	function t:reset()
		self.data = {}
	end
	function t:tostring()
		return table.concat(self.data)
	end
	return t
end

-----------------------------
-- LHTML module
-----------------------------

lhtml = {}

function lhtml.parse2(buf, s)
	while s ~= '' do
		local b, e, pre, code = string.find(s, '^(.-)<%((.-)%)>')
		if b then
			buf:output(string.format('write_raw(%q);', pre))
			buf:output(string.format('write(%s);', code))
			s = string.sub(s, e+1)
		else
			buf:output(string.format('write_raw(%q);', s))
			s = ''
		end
	end
end

function lhtml.parse(s)
	local buf = buffer.create()
	while s ~= '' do
		local b, e, pre, code, nl = string.find(s,'^(.-)<<(.-)>>(\n?)')
		if b then
			lhtml.parse2(buf, pre)
			buf:output(code,' ',nl)
			s = string.sub(s,e+1)
		else
			lhtml.parse2(buf, s)
			s = ''
		end
	end
	return buf:tostring()
end

function lhtml.compile(s)
	local pre =  [[
local buf = buffer.create()
	
local function write_raw(...)
	buf:output(table.concat(arg))
end

local function write(...)
	local xlate_table = {
		['<'] = '&lt;',
		['>'] = '&gt;',
		['&'] = '&amp;',
	}
	
	local function xlate(s)
		return xlate_table[s] or ''
	end
	
	for _, s in ipairs(arg) do
		local s2 = string.gsub(tostring(s), '[<>&]', xlate)
		buf:output(s2)
	end
end
]]
	
	local post = [[
return buf:tostring()
]]

	local s2 = lhtml.parse(s)
	
	return loadstring(pre..s2..post, 'LHTML')
end

---------------------------
-- main
---------------------------

if not fb2k then
	local f, err = lhtml.compile(text)

	total = 5

	if not f then
		error(err)
	else
		local s = f()
		-- in plain Lua 5.0
		io.write(s)
	end
end
