--[[
randgroup.lua

randomizes the playlist; playlist items will be grouped, the order of items in a group will not be changed
]]

-------------------------------------------------
-- options
-------------------------------------------------

-- set your own grouping spec here
local group_by = "%artist%|%album%"

local function group_key_from_handle(handle)
    return handle:format_title(group_by)
end

-------------------------------------------------
-- code
-------------------------------------------------

-- helper
local function print(...)
    -- uncomment the following line to make the script more verbose
    --console.info(table.concat(arg))
end

local function list_groups(tracks)
	local last_key = nil
	local group = nil
    local groups = {}
    for i,track in ipairs(tracks) do
        local key = group_key_from_handle(track)
		if key ~= last_key then
			if group then
				table.insert(groups, group)
			end
			group = {}
			last_key = key
		end
		table.insert(group, track)
    end
	if group then
		table.insert(groups, group)
	end
    return groups
end

local function shuffle_groups(list)
    local res = {}
    local n = table.getn(list)
    while n > 0 do
        local idx = math.random(1,n)
		local group = table.remove(list,idx)
        table.insert(res, group)
        n = n - 1
    end
    return res
end

local function flatten_groups(list)
    local res = {}
    for i,group in ipairs(list) do
        for j,track in ipairs(group) do
            table.insert(res,track)
        end
    end
    return res
end

local function randgroup()
    local groups = {}

    print("getting items")
    -- get items in playlist
    local tracks = playlist.get_data()
    
    print("putting items into groups")
    local groups = list_groups(tracks)
    
    -- initialize random number generator
    math.randomseed(os.time())

    print("shuffling groups")
    groups = shuffle_groups(groups)
    
    print("flattening groups")
    tracks = flatten_groups(groups)
    
    print("clearing playlist")
    playlist.remove_all()
    
    print("adding shuffled items")
    playlist.add_items(tracks,false,-1,false,false)
end

randgroup()
