--[[*
Reloads only technical info from selected files.

@author foosion
@requires foo_lua 0.2
@contextcommand Database/Reload tech info from file(s)
]]

assert(_COMMANDMODE == 'context menu',
	'this script needs to be run from foo_lua as a context menu command')

prefs = {}
local defaults = {}

defaults.preserve_replaygain = 'ask'

defaults.replaygain_tags = {
	'replaygain_track_peak',
	'replaygain_track_gain',
	'replaygain_album_peak',
	'replaygain_album_gain',
}

local function set_defaults()
	for k, v in pairs(defaults) do
		if not prefs[k] then
			prefs[k] = v
		end
	end
end

local function loadprefs()
	dofile(_SCRIPTDIR..'reloadtechinfo-prefs.lua')
end

-- read in preferences
local res, err = pcall(loadprefs)
if not res then
	console.error('error while reading preferences:')
	console.error(err)
	console.info('using default preferences')
end

set_defaults()

-----------------------------
-- code
-----------------------------

-- debug helper
function print(...)
	console.info(table.concat(arg,'\t'))
end

function dialog.ask(question,option)
	if option == 'yes' then
		return true
	elseif option == 'no' then
		return false
	elseif option == 'ask' then
		local d = dialog
		local answer = d.messagebox(question,'Reload tech info',d.MB_YESNO,d.MB_ICONQUESTION)
		return answer == d.IDYES
	else
		error('invalid argument')
	end
end

function reload_tech_info(x, preserve_replaygain)
	local old_info = x:query(true) -- get info from db
	x:flush_info()
	local new_info = x:query() -- get info from file
	local info
	if old_info and new_info then
		-- copy fields that should be preserved
		local preserve = {}
		if preserve_replaygain then
			for _, name in ipairs(prefs.replaygain_tags) do
				local value = old_info:info_get(name)
				if value then
					table.insert(preserve,{name=name,value=value})
				end
			end
		end
		-- remove all info
		old_info:info_remove_all()
		-- copy with keeping item order
		for n = 1, new_info:info_get_count() do
			old_info:info_set(new_info:info_enum(n))
		end
		-- restore preserved fields
		for _, item in ipairs(preserve) do
			old_info:info_set(item.name,item.value)
		end
		info = old_info
	elseif old_info then
		info = old_info
	end
	if info then
		x:update_info(info, true) -- update only database
	end
end

local data

if _COMMANDMODE == 'main menu' then
	data = playlist.get_sel_items()
else
	data = arg[1]
end

if table.getn(data) > 0 then
	local preserve_replaygain = dialog.ask('Should replaygain information be preserved?', prefs.preserve_replaygain)
	for i,v in ipairs(data) do
		reload_tech_info(v, preserve_replaygain)
	end
end
